from ..base import DetectorBaseTest


class TestDetectTV(DetectorBaseTest):

    fixture_files = [
        'tests/fixtures/upstream/tv.yml',
    ]


class TestDetectTV1(DetectorBaseTest):

    fixture_files = [
        'tests/fixtures/upstream/tv-1.yml',
    ]


class TestDetectTV2(DetectorBaseTest):

    fixture_files = [
        'tests/fixtures/upstream/tv-2.yml',
    ]


class TestDetectTV3(DetectorBaseTest):

    fixture_files = [
        'tests/fixtures/upstream/tv-3.yml',
    ]


class TestDetectTV4(DetectorBaseTest):

    fixture_files = [
        'tests/fixtures/upstream/tv-4.yml',
    ]


class TestDetectTV5(DetectorBaseTest):

    fixture_files = [
        'tests/fixtures/upstream/tv-5.yml',
    ]
